
var isRain = false; // This is used for showing the cloud symbol
var isWindy = false; // This is used for showing the wind symbol
var currentTemp = 0; // This holds the value for the current temperature (once it gets fetched from the ElecDesign server)

/**
 * This function fetches the data (JSON) from the ElecDesign server.
 * The function is called by the 'drawChart()' function further below.
 * The 'async' means that the code will run on the background, and then 
 * get back, when it has the result.
 * 
 * You don't need to understand this part beyond that.
 */
async function getData() {
    result = await fetch("https://elecdesign.org.aalto.fi/nr1/weatherjson")
        .then(response => response.json());
    return result;
}

/**
 * This function parses the JSON data. 
 * Central to this function is the 'for loop'.
 * It goes through all the elements in the data, and then puts the values to local variables. 
 */
function parseChartData(weatherData) {
    let timeSeries = { times: [], temps: [], rains: [] };
    for (let i=0; i<weatherData.length; i++) {
        let instant = { time: '', temperature: '', rain: '', windSpeed: '', windDirection: '' }
        // [3, 11, 2020, 23, 10.1, 0, 3.9, 246]
        let dateString = "";
        let month = '0';
        let day = '0';
        let hour = '0';
        let year = weatherData[i][2];
        
        if (weatherData[i][0]<10)  //if day is one digit, add 0 in front
            day = day + weatherData[i][0];
        else day = weatherData[i][0];
        if (weatherData[i][1]<10)   // month
            month = month + weatherData[i][1];
        else month = weatherData[i][1];
        if (weatherData[i][3]<10)   // hour
            hour = hour + weatherData[i][3];
        else hour = weatherData[i][3];
        dateString = day + '.' + month + '.' + year + ' ' + hour + ':00';
                    
        let temp = weatherData[i][4];
        let rain =  weatherData[i][5];
        let windSpeed =  weatherData[i][6];
        let windDirection =  weatherData[i][7];

        if (windSpeed > 6 && i <9) { // the 9 means that we are within 8 hours into the forecast data
            isWindy = true;
        }
        if (rain > 0 && i <9) { 
            isRain = true;
        }
        if (i==0) {
            if (temp > 0) {
                currentTemp = '+' + temp + '°C';
            } else {
                currentTemp = temp + '°C';
            }
        }

        timeSeries.times.push(dateString);
        timeSeries.temps.push(temp);
        timeSeries.rains.push(rain);
    }
    return timeSeries;
}

/**
 * This function draws the chart. It first asks to get the data from ElecDesign server with the getData() call.
 * Then it sends the weatherData to be processed in the parseChartData() fuction,
 * which returns 'chartData'. 
 * The 'chartData' is a data structure, which the Charts.js script is able to use. 
 * Chart.js is the external library that we use for creating the visual graphs.
 * 
 * In the very end, once all is complete with the data, the drawSymbols() is called.
 */
async function drawChart() {
    let weatherData = await getData();
    let chartData = parseChartData(weatherData);

    //console.log(chartData);

    var timeFormat = 'DD.MM.YYYY HH:mm';

    Chart.defaults.global.elements.point.radius = 0;

    var ctx = 'myChart';

    var myChart = new Chart(ctx, {
        type: 'bar',
        data: {
            labels: chartData.times,
            datasets: [{
                yAxisID: 'temp-axis',
                type: 'line',
                label: 'Temperatures',
                backgroundColor: 'rgba(180, 5, 5, 0.05)',
                borderColor: 'rgba(180, 5, 5, 0.8)',
                borderWidth: 2,
                fill: false,
                data: chartData.temps
            },{
                yAxisID: 'rain-axis',
                type: 'bar',
                label: 'Rain',
                backgroundColor: 'rgba(5, 5, 225, 0.05)',
                borderColor: 'rgba(5, 5, 225, 0.5)',
                borderWidth: 1,
                fill: true,
                data: chartData.rains
            }]
        },
        options: {
            responsive: false,
            legend: {
                display: false
            },
            scales: {
                xAxes: [{
                    type: 'time',
                    time: {
                        displayFormats: {
                            day: 'DD.MM.',
                            hour: 'HH:mm'
                        },
                        parser: timeFormat,
                        stepSize: 4,
                        minUnit: 'hour'
                    }
                },{
                    type: 'time',
                    major: {
                        display: true
                    },
                    gridLines: {
                        display: true
                    },
                    time: {
                        displayFormats: {
                            day: 'dddd',
                            hour: 'HH:mm'
                        },
                        parser: timeFormat,
                        stepSize: 1,
                        minUnit: 'day'
                    }
                }],
                yAxes: [{
                    scaleLabel: {
                        labelString: 'Temperature °C',
                        fontColor: 'rgba(180, 5, 5, 0.8)',
                        display: true
                    },
                    id: 'temp-axis',
                    type: 'linear'
                    /*ticks: {
                        min: -10,
                        max: 20
                    }*/
                },{
                    scaleLabel: {
                        labelString: 'Rain (mm)',
                        fontColor: 'rgba(5, 5, 225, 0.5)',
                        display: true
                    },
                    gridLines: {
                        display: false
                    },
                    id: 'rain-axis',
                    position: 'right',
                    type: 'linear',
                    ticks: { 
                        suggestedMin: 0,
                        suggestedMax: 2
                    }
                }]
            },
        }
    });

    drawSymbols();

}
/**
 * This function sets the visibility of the rain and wind symbols. 
 * It simply changes the 'style' of those elements in the html document object model. 
 */
function drawSymbols() {
    if (isWindy) 
        document.getElementById('wind-symbol').style.visibility = 'visible';
    
    if (isRain)
        document.getElementById('rain-symbol').style.visibility = 'visible';

    document.getElementById('temp-container').innerHTML = currentTemp;
}

/**
 * This is a function call. It calls the drawChart() function, which begins the whole process.
 * Everythin else, i.e. other functions, becomes called from drawChart().
 */
drawChart();

